/* bipalea.c   graphe biparti alatoire */

// *INDENT-OFF* 
/* bipalea.c
 *
 * Gnre un graphe biparti G(X, Y, E) alatoire pour une utilisation 
 * ultrieure par le progamme affect (affect.c) qui cherche tous les 
 * couplages de X dans Y.  |X| <= |Y|    k=|X|, n=|Y|
 * On peut supposer que le graphe est orient de x in X vers y in Y
 * et le programme cherchera  avoir deg_+(x)>= 1 pour tout x in X et 
 * deg_-(y) >= 1 pour au moins k lments de Y
 * Par dfaut n=k ou k=n (suivant l'option indique en ligne de commande)
 * 
 * taux de remplissage minimum garanti
 * 0< taux <1 par dfaut taux = 0.25 tel que
 *          [ taux * |X| * |Y| ] <= |E|
 * Le taux effectif est en ralit parfois nettement suprieur au taux minimum 
 * lorsque celui-ci ne permet pas des degrs 1 minimum.
 * Option non encore implmente mais prvue :  mini <= deg+(x) <= maxi)
 * compilation : gcc -O2 -o bipalea bipalea.c -lm 
 * usage :  bipalea -k # [-n #] [-t #] | affect
 * exemple : bipalea -k 10 -t 0.3 | affect
 * 29900 Concarneau (France) le : 03 Mai 2006
 *
 * $Modified:
 * 
 * bipalea.c, Copyright (C) 2006 Jean-Paul Davalan.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Neuron; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Jean-Paul Davalan <jeanpaul.davalan@wanadoo.fr>a
 *
 * indent -br -ce -cdw -cli2 -cbi0 -ss -cs -bs -saf -nbc -psl -brs -ip2 -cdb bipalea.c
 */
//  *INDENT-ON*

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#define VERSION "version 0.0"

#define max(U,V) ((U>V)?U:V)
#define min(U,V) ((U<V)?U:V)


/* Options */
int k = -1, n = -1, mini = -1, maxi = -1;
double taux = 0.25;

static char *helparr[] = {
  "Options :",
  "  --help         -h",
  "  --version      -v",
  "  --copyright    -c",
  "  --k=k          -k k",
  "  --n=n          -n n",
  "  --min=min      -m min",
  "  --max=Max      -M Max",
  "  --taux=t       -t t 0<t<1",
  0
};

static const struct option long_options[] = {
  {"help", no_argument, 0, 'h'},
  {"version", no_argument, 0, 'v'},
  {"copyright", no_argument, 0, 'c'},
  {"k", required_argument, 0, 'k'},
  {"n", required_argument, 0, 'n'},
  {"min", required_argument, 0, 'm'},
  {"max", required_argument, 0, 'M'},
  {"taux", required_argument, 0, 't'},
  {0, 0, 0}
};

void options ();
void help ();
void copyright ();
void init_rand (void);
int choix (int);
void principal ();

int
main (int argc, char *argv[])
{
  options (argc, argv);
  init_rand ();
  principal ();
  return 0;
}

void
options (int argc, char *argv[])
{
  int c, longoptindex = 0, errflag = 0;
  if (argc == 1) {
    help (argc, argv);
    exit (2);
  }
  while ((c = getopt_long_only (argc, argv,
				"hvck:n:m:M:t:",
				long_options, &longoptindex)) != EOF) {
    switch (c) {
      case 0:
	switch (longoptindex) {
	  case 1:
	    break;
	  default:
	    break;
	}
	break;
      case 'h':
	help (argc, argv);
	exit (1);
	break;
      case 'v':
	printf ("%s\n", VERSION);
	break;
      case 'c':
	copyright (argc, argv);
	exit (1);
	break;
      case 'k':
	if (optarg) {
	  k = atoi (optarg);
	}
	break;
      case 'n':
	if (optarg) {
	  n = atoi (optarg);
	}
	break;
      case 'm':
	if (optarg) {
	  mini = atoi (optarg);
	}
	break;
      case 'M':
	if (optarg) {
	  maxi = atoi (optarg);
	}
	break;
      case 't':
	if (optarg) {
	  taux = atof (optarg);
	}
	break;
      default:
	errflag++;
	break;
    }
  }
  if (errflag) {
    help (argc, argv);
    exit (2);
  }
  if (k == -1 && n == -1) {
    k = n = 5;
  } else if (k == -1) {
    k = n;
  } else if (n == -1) {
    n = k;
  } else if (k < n) {
    n = k;
  }
  if (taux <= 0 || taux >= 1) {
    taux = 0.25;
  }
  if (mini < 1)
    mini = 1;
  if (maxi > n)
    maxi = n;
}

/*---------------------------------------------------------------*/
void
help (int argc, char *argv[])
{
  char **pt;
  fprintf (stderr, "usage: %s options\n", argv[0]);
  for (pt = helparr; *pt; pt++)
    fprintf (stderr, "%s\n", *pt);
}

/*---------------------------------------------------------------*/
void
copyright (int argc, char *argv[])
{
  fprintf (stdout,
	   " This application %s , Copyright (C) 2006 Jean-Paul Davalan.\n\n"
	   " 29900 Concarneau (France)\n\n"
	   " This application %s is free software; you can redistribute it and/or modify\n"
	   " it under the terms of the GNU General Public License as published by\n"
	   " the Free Software Foundation; either version 2, or (at your option)\n"
	   " any later version.\n\n"
	   " This application %s is distributed in the hope that it will be useful,\n"
	   " but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	   " MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
	   " GNU General Public License for more details.\n"
	   " You should have received a copy of the GNU General Public License\n"
	   " along with %s (Neuron); see the file COPYING.  If not, write to\n"
	   " the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"
	   " Jean-Paul Davalan <jpdvl@wanadoo.fr>\n", argv[0], argv[0],
	   argv[0], argv[0]);
}

/*---------------------------------------------------------------*/

void
init_rand (void)
{
  srand ((unsigned int) (time (NULL) * getpid ()));
}

/*---------------------------------------------------------------
  retourne une valeur entre 0 et r
*/

int
choix (int r)
{
  return (int) (((double) r * (double) rand ()) / (RAND_MAX + 1.0));
}

/*---------------------------------------------------------------

  */

void
principal ()
{
  int i, j, tot = floor (k * n * taux);
  int *Matr[k], L[k], C[n], nL = k, nC = n, nb = 0;
  for (i = 0; i < k; i++) {
    L[i] = 0;
    Matr[i] = (int *) malloc (n * sizeof (int));
    for (j = 0; j < n; j++) {
      Matr[i][j] = 0;
    }
  }
  for (j = 0; j < n; j++) {
    C[j] = 0;
  }
  do {
    i = (int) choix (k);
    j = (int) choix (n);
    j = (int) choix (n);
    if (Matr[i][j] == 0) {
      nb++;
      if (L[i] == 0)
	nL--;
      if (C[j] == 0)
	nC--;
      L[i] = 1;
      C[j] = 1;
      Matr[i][j] = 1;
    }
  }
  while (nC+k > n || nL > 0 || nb < tot);
  printf ("%d %d\n", k, n);
  for (i = 0; i < k; i++) {
    int d = 0;
    for (j = 0; j < n; j++)
      d += Matr[i][j];
    printf ("%d %d ", i, d);
    for (j = 0; j < n; j++) {
      if (Matr[i][j] == 1) {
	printf (" %d", j);
      }
    }
    printf ("\n");
  }
  for (i = 0; i < k; i++)
    free (Matr[i]);
}

/*---------------------------------------------------------------*/
